searchData={"items":[{"type":"module","title":"Lapin","doc":"Lapin, an Elixir RabbitMQ client\n\nComprehensive documentation is available [here](main.html).","ref":"Lapin.html"},{"type":"type","title":"Lapin.config/0","doc":"Lapin configuration","ref":"Lapin.html#t:config/0"},{"type":"behaviour","title":"Lapin.Connection","doc":"RabbitMQ connection handler\n\nThis module handles the RabbitMQ connection. It also provides a behaviour for\nworker module implementation. The worker module should use the `Lapin.Connection`\nbehaviour and implement the callbacks it needs.\n\nWhen using the `Lapin.Connection` behaviour a `publish/4` function is injected in\nthe worker module as a shortcut to the `Lapin.Connection.publish/5` function\nwhich removes the need for passing in the connection and is publicly callable\nto publish messages on the connection configured for the implementing module.","ref":"Lapin.Connection.html"},{"type":"function","title":"Lapin.Connection.close/1","doc":"Closes the connection","ref":"Lapin.Connection.html#close/1"},{"type":"callback","title":"Lapin.Connection.handle_cancel/1","doc":"Called when receiving a `basic.cancel` from the broker.","ref":"Lapin.Connection.html#c:handle_cancel/1"},{"type":"callback","title":"Lapin.Connection.handle_cancel_ok/1","doc":"Called when receiving a `basic.cancel_ok` from the broker.","ref":"Lapin.Connection.html#c:handle_cancel_ok/1"},{"type":"callback","title":"Lapin.Connection.handle_consume_ok/1","doc":"Called when receiving a `basic.consume_ok` from the broker.\n\nThis signals successul registration as a consumer.","ref":"Lapin.Connection.html#c:handle_consume_ok/1"},{"type":"callback","title":"Lapin.Connection.handle_deliver/2","doc":"Called when receiving a `basic.deliver` from the broker.\n\nReturn values from this callback determine message acknowledgement:\n  - `:ok`: Message was processed by the consumer and should be removed from queue\n  - `{:reject, reason}`: Message was not processed and should be rejected\n\nAny other return value requeues the message to prevent data loss.\nA crash in the callback code will however reject the message to prevent loops\nif the message was already delivered before.\n\nThe `reason` term can be used by the application\nto signal the reason of rejection and is logged in debug.","ref":"Lapin.Connection.html#c:handle_deliver/2"},{"type":"callback","title":"Lapin.Connection.handle_publish/2","doc":"Called when completing a `basic.publish` with the broker.\n\nMessage transmission to the broker is successful when this callback is called.","ref":"Lapin.Connection.html#c:handle_publish/2"},{"type":"callback","title":"Lapin.Connection.handle_return/2","doc":"Called when receiving a `basic.return` from the broker.\n\nThis signals an undeliverable returned message from the broker.","ref":"Lapin.Connection.html#c:handle_return/2"},{"type":"callback","title":"Lapin.Connection.payload_for/2","doc":"Called before `handle_deliver/2` to get the payload type.\n\nShould return a data type instance to decode the payload into.\nA `Lapin.Message.Payload` implementation must be provided for this type. The\ndefault implementation leaves the payload unaltered.","ref":"Lapin.Connection.html#c:payload_for/2"},{"type":"function","title":"Lapin.Connection.publish/5","doc":"Publishes a message to the specified exchange with the given routing_key","ref":"Lapin.Connection.html#publish/5"},{"type":"function","title":"Lapin.Connection.start_link/2","doc":"Starts a `Lapin.Connection` with the specified configuration","ref":"Lapin.Connection.html#start_link/2"},{"type":"type","title":"Lapin.Connection.config/0","doc":"Connection configuration\n\nThe following keys are supported:\n  - module: module using the `Lapin.Connection` behaviour\n  - uri: AMQP URI (String.t | URI.t)\n  - host: broker hostname (string | charlist), *default: 'localhost'*\n  - port: broker port (string | integer), *default: 5672*\n  - virtual_host: broker vhost (string), *default: \"/\"*\n  - username: username (string)\n  - password: password (string)\n  - auth_mechanisms: broker auth_mechanisms ([:amqplain | :external | :plain]), *default: amqp_client default*\n  - ssl_options: ssl options ([:ssl:ssl_option]), *default: none*\n  - producers: producers to configure ([Producer.config]), *default: []*\n  - consumers: consumers to configure ([Consumer.config]), *default: []*","ref":"Lapin.Connection.html#t:config/0"},{"type":"type","title":"Lapin.Connection.on_callback/0","doc":"Callback result","ref":"Lapin.Connection.html#t:on_callback/0"},{"type":"type","title":"Lapin.Connection.on_deliver/0","doc":"`handle_deliver/2` callback result","ref":"Lapin.Connection.html#t:on_deliver/0"},{"type":"type","title":"Lapin.Connection.reason/0","doc":"Reason for message rejection","ref":"Lapin.Connection.html#t:reason/0"},{"type":"type","title":"Lapin.Connection.t/0","doc":"Connection","ref":"Lapin.Connection.html#t:t/0"},{"type":"module","title":"Lapin.Connection.Supervisor","doc":"Lapin Connections Supervisor","ref":"Lapin.Connection.Supervisor.html"},{"type":"function","title":"Lapin.Connection.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Lapin.Connection.Supervisor.html#child_spec/1"},{"type":"function","title":"Lapin.Connection.Supervisor.init/1","doc":"","ref":"Lapin.Connection.Supervisor.html#init/1"},{"type":"function","title":"Lapin.Connection.Supervisor.start_link/1","doc":"","ref":"Lapin.Connection.Supervisor.html#start_link/1"},{"type":"behaviour","title":"Lapin.Consumer","doc":"Extensible behaviour to define consumer configuration.\n\nLapin provides a number of submodules which implement the patterns found in\nthe [RabbitMQ Tutorials](http://www.rabbitmq.com/getstarted.html).\n\n```\ndefmodule ExampleApp.SomeConsumer do\n  use Lapin.Consumer\n\n  [... callbacks implementation ...]\nend\n```","ref":"Lapin.Consumer.html"},{"type":"callback","title":"Lapin.Consumer.ack/1","doc":"Consumer acknowledgements enabled","ref":"Lapin.Consumer.html#c:ack/1"},{"type":"function","title":"Lapin.Consumer.ack_message/2","doc":"ACK message consumption","ref":"Lapin.Consumer.html#ack_message/2"},{"type":"function","title":"Lapin.Consumer.create/2","doc":"Creates a consumer from configuration","ref":"Lapin.Consumer.html#create/2"},{"type":"function","title":"Lapin.Consumer.get/2","doc":"Find consumer by consumer_tag","ref":"Lapin.Consumer.html#get/2"},{"type":"callback","title":"Lapin.Consumer.prefetch_count/1","doc":"Consumer message prefetch count","ref":"Lapin.Consumer.html#c:prefetch_count/1"},{"type":"callback","title":"Lapin.Consumer.queue/1","doc":"Queue to consume from","ref":"Lapin.Consumer.html#c:queue/1"},{"type":"function","title":"Lapin.Consumer.reject_message/3","doc":"Reject message","ref":"Lapin.Consumer.html#reject_message/3"},{"type":"type","title":"Lapin.Consumer.config/0","doc":"Consumer configuration\n\nThe following keys are supported:\n  - pattern: producer pattern (module using the `Lapin.Producer` behaviour)\n\nIf using the `Lapin.Consumer.Config` default implementation, the following keys are also supported:\n  - queue: queue to consume from, (`String.t()`, *required*)\n  - ack: producer ack (`boolean()`, default: false*\n  - prefetch_count: consumer prefetch count (`integer()`, *default: 1*)","ref":"Lapin.Consumer.html#t:config/0"},{"type":"type","title":"Lapin.Consumer.consumer_tag/0","doc":"Consumer Tag","ref":"Lapin.Consumer.html#t:consumer_tag/0"},{"type":"type","title":"Lapin.Consumer.prefetch_count/0","doc":"Consumer Prefetch","ref":"Lapin.Consumer.html#t:prefetch_count/0"},{"type":"type","title":"Lapin.Consumer.t/0","doc":"Lapin Consumer Behaviour","ref":"Lapin.Consumer.html#t:t/0"},{"type":"module","title":"Lapin.Consumer.Config","doc":"Lapin Consumer default pattern, reads settings from configuration","ref":"Lapin.Consumer.Config.html"},{"type":"function","title":"Lapin.Consumer.Config.ack/1","doc":"","ref":"Lapin.Consumer.Config.html#ack/1"},{"type":"function","title":"Lapin.Consumer.Config.prefetch_count/1","doc":"","ref":"Lapin.Consumer.Config.html#prefetch_count/1"},{"type":"function","title":"Lapin.Consumer.Config.queue/1","doc":"","ref":"Lapin.Consumer.Config.html#queue/1"},{"type":"module","title":"Lapin.Consumer.PubSub","doc":"`Lapin.Consumer` implementation for the\n[Publish/Subscribe](http://www.rabbitmq.com/tutorials/tutorial-three-elixir.html)\nRabbitMQ pattern.","ref":"Lapin.Consumer.PubSub.html"},{"type":"function","title":"Lapin.Consumer.PubSub.ack/1","doc":"","ref":"Lapin.Consumer.PubSub.html#ack/1"},{"type":"function","title":"Lapin.Consumer.PubSub.prefetch_count/1","doc":"","ref":"Lapin.Consumer.PubSub.html#prefetch_count/1"},{"type":"function","title":"Lapin.Consumer.PubSub.queue/1","doc":"","ref":"Lapin.Consumer.PubSub.html#queue/1"},{"type":"module","title":"Lapin.Consumer.Routing","doc":"`Lapin.Consumer` implementation for the\n[Routing](http://www.rabbitmq.com/tutorials/tutorial-four-elixir.html)\nRabbitMQ pattern.","ref":"Lapin.Consumer.Routing.html"},{"type":"function","title":"Lapin.Consumer.Routing.ack/1","doc":"","ref":"Lapin.Consumer.Routing.html#ack/1"},{"type":"function","title":"Lapin.Consumer.Routing.prefetch_count/1","doc":"","ref":"Lapin.Consumer.Routing.html#prefetch_count/1"},{"type":"function","title":"Lapin.Consumer.Routing.queue/1","doc":"","ref":"Lapin.Consumer.Routing.html#queue/1"},{"type":"module","title":"Lapin.Consumer.Topic","doc":"`Lapin.Consumer` implementation for the\n[Topics](http://www.rabbitmq.com/tutorials/tutorial-five-elixir.html)\nRabbitMQ pattern.","ref":"Lapin.Consumer.Topic.html"},{"type":"function","title":"Lapin.Consumer.Topic.ack/1","doc":"","ref":"Lapin.Consumer.Topic.html#ack/1"},{"type":"function","title":"Lapin.Consumer.Topic.prefetch_count/1","doc":"","ref":"Lapin.Consumer.Topic.html#prefetch_count/1"},{"type":"function","title":"Lapin.Consumer.Topic.queue/1","doc":"","ref":"Lapin.Consumer.Topic.html#queue/1"},{"type":"module","title":"Lapin.Consumer.WorkQueue","doc":"`Lapin.Consumer` implementation for the\n[Work Queues](http://www.rabbitmq.com/tutorials/tutorial-two-elixir.html)\nRabbitMQ pattern.","ref":"Lapin.Consumer.WorkQueue.html"},{"type":"function","title":"Lapin.Consumer.WorkQueue.ack/1","doc":"","ref":"Lapin.Consumer.WorkQueue.html#ack/1"},{"type":"function","title":"Lapin.Consumer.WorkQueue.prefetch_count/1","doc":"","ref":"Lapin.Consumer.WorkQueue.html#prefetch_count/1"},{"type":"function","title":"Lapin.Consumer.WorkQueue.queue/1","doc":"","ref":"Lapin.Consumer.WorkQueue.html#queue/1"},{"type":"module","title":"Lapin.Exchange","doc":"Lapin Exchange","ref":"Lapin.Exchange.html"},{"type":"function","title":"Lapin.Exchange.bind/2","doc":"","ref":"Lapin.Exchange.html#bind/2"},{"type":"function","title":"Lapin.Exchange.declare/2","doc":"","ref":"Lapin.Exchange.html#declare/2"},{"type":"function","title":"Lapin.Exchange.new/2","doc":"","ref":"Lapin.Exchange.html#new/2"},{"type":"type","title":"Lapin.Exchange.name/0","doc":"","ref":"Lapin.Exchange.html#t:name/0"},{"type":"type","title":"Lapin.Exchange.routing_key/0","doc":"","ref":"Lapin.Exchange.html#t:routing_key/0"},{"type":"type","title":"Lapin.Exchange.t/0","doc":"Exchange","ref":"Lapin.Exchange.html#t:t/0"},{"type":"type","title":"Lapin.Exchange.type/0","doc":"","ref":"Lapin.Exchange.html#t:type/0"},{"type":"module","title":"Lapin.Message","doc":"Lapin Message struct","ref":"Lapin.Message.html"},{"type":"type","title":"Lapin.Message.meta/0","doc":"Message metadata","ref":"Lapin.Message.html#t:meta/0"},{"type":"type","title":"Lapin.Message.payload/0","doc":"Message payload","ref":"Lapin.Message.html#t:payload/0"},{"type":"type","title":"Lapin.Message.t/0","doc":"Lapin Message container","ref":"Lapin.Message.html#t:t/0"},{"type":"protocol","title":"Lapin.Message.Payload","doc":"You can use this protocol to implement a custom message payload transformation.\nFor example you could implement a JSON message with a predefined structure by\nfirst implementing a struct for your payload:\n\n```elixir\ndefmodule Example.Payload do\n  defstruct [a: \"a\", b: \"b\", c: nil]\nend\n```\n\nand then providing an implementation of `Lapin.Message.Payload` for it:\n\n```elixir\ndefimpl Lapin.Message.Payload, for: Example.Payload do\n  def content_type(_payload), do: \"application/json\"\n  def encode(payload), do: Poison.encode(payload)\n  def decode_into(payload, data), do: Poison.decode(data, as: payload)\nend\n```\n\nPlease note you will need to add the `poison` library as a dependency on in\nyour project `mix.exs` for this to work.\n\nLapin will automatically encode and set the `content-type` property on publish.\n\nTo decode messages before consuming, implement the `payload_for/2` callback\nof `Lapin.Connection` and return an instance of the payload to decode into.\n\n```elixir\ndefmodule Example.Connection do\n  def payload_for(_channel, _message), do: %Example.Payload{}\nend\n```\n\nThe default implementation simply returns the unaltered binary data and sets\nthe message `content-type` property to `nil`.","ref":"Lapin.Message.Payload.html"},{"type":"function","title":"Lapin.Message.Payload.content_type/1","doc":"Returns the message `content-type`","ref":"Lapin.Message.Payload.html#content_type/1"},{"type":"function","title":"Lapin.Message.Payload.decode_into/2","doc":"Returns the payload with message data decoded","ref":"Lapin.Message.Payload.html#decode_into/2"},{"type":"function","title":"Lapin.Message.Payload.encode/1","doc":"Returns the encoded payload body","ref":"Lapin.Message.Payload.html#encode/1"},{"type":"type","title":"Lapin.Message.Payload.content_type/0","doc":"MIME content-type as defined by RFC 2045","ref":"Lapin.Message.Payload.html#t:content_type/0"},{"type":"type","title":"Lapin.Message.Payload.on_decode/0","doc":"Decode function return values","ref":"Lapin.Message.Payload.html#t:on_decode/0"},{"type":"type","title":"Lapin.Message.Payload.on_encode/0","doc":"Encode function return values","ref":"Lapin.Message.Payload.html#t:on_encode/0"},{"type":"type","title":"Lapin.Message.Payload.t/0","doc":"Data type implementing the `Lapin.Message.Payload` protocol","ref":"Lapin.Message.Payload.html#t:t/0"},{"type":"behaviour","title":"Lapin.Producer","doc":"Extensible behaviour to define producer configuration.\n\nLapin provides a number of submodules which implement the patterns found in\nthe [RabbitMQ Tutorials](http://www.rabbitmq.com/getstarted.html).\n\n```\ndefmodule ExampleApp.SomePatter do\n  use Lapin.Producer\n\n  [... callbacks implementation ...]\nend\n```","ref":"Lapin.Producer.html"},{"type":"function","title":"Lapin.Producer.confirm/1","doc":"Wait for publish confirmation","ref":"Lapin.Producer.html#confirm/1"},{"type":"callback","title":"Lapin.Producer.confirm/1","doc":"Request publisher confirms (RabbitMQ only)","ref":"Lapin.Producer.html#c:confirm/1"},{"type":"function","title":"Lapin.Producer.create/2","doc":"Creates a producer from configuration","ref":"Lapin.Producer.html#create/2"},{"type":"callback","title":"Lapin.Producer.exchange/1","doc":"Declare exchange","ref":"Lapin.Producer.html#c:exchange/1"},{"type":"function","title":"Lapin.Producer.get/2","doc":"Find consumer by consumer_tag","ref":"Lapin.Producer.html#get/2"},{"type":"callback","title":"Lapin.Producer.mandatory/1","doc":"Request message mandatory routing when publishing","ref":"Lapin.Producer.html#c:mandatory/1"},{"type":"callback","title":"Lapin.Producer.persistent/1","doc":"Request message persistence when publishing","ref":"Lapin.Producer.html#c:persistent/1"},{"type":"function","title":"Lapin.Producer.publish/5","doc":"Publish message","ref":"Lapin.Producer.html#publish/5"},{"type":"type","title":"Lapin.Producer.config/0","doc":"Producer configuration\n\nThe following keys are supported:\n  - pattern: producer pattern (module using the `Lapin.Producer` behaviour)\n\nIf using the `Lapin.Producer.Config` default implementation, the following keys are also supported:\n  - exchange: exchange used for publish (`String.t`, *required*)\n  - confirm: expect RabbitMQ publish confirms (`boolean()`, *default: false*)\n  - mandatory: messages published as mandatory by default (`boolean()`, *default: false*)\n  - persistent: messages published as persistent by default (`boolean()`, *default: false*)","ref":"Lapin.Producer.html#t:config/0"},{"type":"type","title":"Lapin.Producer.t/0","doc":"Lapin Producer","ref":"Lapin.Producer.html#t:t/0"},{"type":"module","title":"Lapin.Producer.Config","doc":"Lapin Producer default pattern, reads settings from configuration","ref":"Lapin.Producer.Config.html"},{"type":"function","title":"Lapin.Producer.Config.confirm/1","doc":"","ref":"Lapin.Producer.Config.html#confirm/1"},{"type":"function","title":"Lapin.Producer.Config.exchange/1","doc":"","ref":"Lapin.Producer.Config.html#exchange/1"},{"type":"function","title":"Lapin.Producer.Config.mandatory/1","doc":"","ref":"Lapin.Producer.Config.html#mandatory/1"},{"type":"function","title":"Lapin.Producer.Config.persistent/1","doc":"","ref":"Lapin.Producer.Config.html#persistent/1"},{"type":"module","title":"Lapin.Producer.PubSub","doc":"`Lapin.Producer` implementation for the\n[Publish/Subscribe](http://www.rabbitmq.com/tutorials/tutorial-three-elixir.html)\nRabbitMQ pattern.","ref":"Lapin.Producer.PubSub.html"},{"type":"function","title":"Lapin.Producer.PubSub.confirm/1","doc":"","ref":"Lapin.Producer.PubSub.html#confirm/1"},{"type":"function","title":"Lapin.Producer.PubSub.exchange/1","doc":"","ref":"Lapin.Producer.PubSub.html#exchange/1"},{"type":"function","title":"Lapin.Producer.PubSub.mandatory/1","doc":"","ref":"Lapin.Producer.PubSub.html#mandatory/1"},{"type":"function","title":"Lapin.Producer.PubSub.persistent/1","doc":"","ref":"Lapin.Producer.PubSub.html#persistent/1"},{"type":"module","title":"Lapin.Producer.Routing","doc":"`Lapin.Producer` implementation for the\n[Routing](http://www.rabbitmq.com/tutorials/tutorial-four-elixir.html)\nRabbitMQ pattern.","ref":"Lapin.Producer.Routing.html"},{"type":"function","title":"Lapin.Producer.Routing.confirm/1","doc":"","ref":"Lapin.Producer.Routing.html#confirm/1"},{"type":"function","title":"Lapin.Producer.Routing.exchange/1","doc":"","ref":"Lapin.Producer.Routing.html#exchange/1"},{"type":"function","title":"Lapin.Producer.Routing.mandatory/1","doc":"","ref":"Lapin.Producer.Routing.html#mandatory/1"},{"type":"function","title":"Lapin.Producer.Routing.persistent/1","doc":"","ref":"Lapin.Producer.Routing.html#persistent/1"},{"type":"module","title":"Lapin.Producer.Topic","doc":"`Lapin.Producer` implementation for the\n[Topics](http://www.rabbitmq.com/tutorials/tutorial-five-elixir.html)\nRabbitMQ pattern.","ref":"Lapin.Producer.Topic.html"},{"type":"function","title":"Lapin.Producer.Topic.confirm/1","doc":"","ref":"Lapin.Producer.Topic.html#confirm/1"},{"type":"function","title":"Lapin.Producer.Topic.exchange/1","doc":"","ref":"Lapin.Producer.Topic.html#exchange/1"},{"type":"function","title":"Lapin.Producer.Topic.mandatory/1","doc":"","ref":"Lapin.Producer.Topic.html#mandatory/1"},{"type":"function","title":"Lapin.Producer.Topic.persistent/1","doc":"","ref":"Lapin.Producer.Topic.html#persistent/1"},{"type":"module","title":"Lapin.Producer.WorkQueue","doc":"`Lapin.Producer` implementation for the\n[Work Queues](http://www.rabbitmq.com/tutorials/tutorial-two-elixir.html)\nRabbitMQ pattern.","ref":"Lapin.Producer.WorkQueue.html"},{"type":"function","title":"Lapin.Producer.WorkQueue.confirm/1","doc":"","ref":"Lapin.Producer.WorkQueue.html#confirm/1"},{"type":"function","title":"Lapin.Producer.WorkQueue.exchange/1","doc":"","ref":"Lapin.Producer.WorkQueue.html#exchange/1"},{"type":"function","title":"Lapin.Producer.WorkQueue.mandatory/1","doc":"","ref":"Lapin.Producer.WorkQueue.html#mandatory/1"},{"type":"function","title":"Lapin.Producer.WorkQueue.persistent/1","doc":"","ref":"Lapin.Producer.WorkQueue.html#persistent/1"},{"type":"module","title":"Lapin.Queue","doc":"Lapin Queue","ref":"Lapin.Queue.html"},{"type":"function","title":"Lapin.Queue.bind/2","doc":"","ref":"Lapin.Queue.html#bind/2"},{"type":"function","title":"Lapin.Queue.declare/2","doc":"","ref":"Lapin.Queue.html#declare/2"},{"type":"function","title":"Lapin.Queue.new/2","doc":"","ref":"Lapin.Queue.html#new/2"},{"type":"type","title":"Lapin.Queue.t/0","doc":"Queue","ref":"Lapin.Queue.html#t:t/0"},{"type":"extras","title":"Lapin, a RabbitMQ client for Elixir","doc":"# Lapin, a RabbitMQ client for Elixir","ref":"main.html"},{"type":"extras","title":"Description - Lapin, a RabbitMQ client for Elixir","doc":"**Lapin** is a *RabbitMQ* client for *Elixir* which abstracts away a lot of the\ncomplexities of interacting with an *AMQP* broker.\n\nWhile some advanced features, like [publisher confirms](http://www.rabbitmq.com/confirms.html),\nare tied to *RabbitMQ* implementation specific extensions, **Lapin** should play\nwell with other broker implementations conforming to the AMQP 0.9.1 specification.","ref":"main.html#description"},{"type":"extras","title":"Installation - Lapin, a RabbitMQ client for Elixir","doc":"Just add **Lapin** as a dependency to your `mix.exs`:\n\n```elixir\ndefp deps() do\n  [{:lapin, \"~> 1.0.0\"}]\nend\n```\n\nAnd add the `Lapin.Supervisor` under your application supervision tree:\n\n```elixir\nchildren = [\n  Lapin.Supervisor\n  ...\n]\n```","ref":"main.html#installation"},{"type":"extras","title":"Quick Start - Lapin, a RabbitMQ client for Elixir","doc":"If you are impatient to try **Lapin** out, just tweak this basic configuration\nexample:\n\n```elixir\nconfig :lapin, :connections, [\n  [\n    module: ExampleApp.Worker,\n    consumers: [\n      [queue: \"some_queue\"]\n    ],\n    producers: [\n      [exchange: \"some_exchange\"]\n    ]\n  ]\n]\n```\n\nand define your worker module as follows:\n\n```elixir\ndefmodule ExampleApp.Worker do\n  use Lapin.Connection\nend\n```\n\nTo test your setup make sure *RabbitMQ* is running and configured correctly, then\nrun your application with `iex -S mix` and publish a message:\n\n```elixir\n...\niex(1)> ExampleApp.Worker.publish(\"some_exchange\", \"routing_key\", \"payload\")\nPublished %Lapin.Message{meta: %{content_type: nil, mandatory: true, persistent: true}, payload: \"msg\"} on %Lapin.Producer{channel: %AMQP.Channel{conn: %AMQP.Connection{pid: #PID }, pid: #PID }, config: [pattern: Lapin.Producer.WorkQueue, exchange: \"some_exchange\"], exchange: \"some_exchange\", pattern: Lapin.Producer.WorkQueue}\n:ok\n[debug] Consuming message 1\n[debug] Consumed message 1 successfully, ACK sent\n...\n```\n\nRead on to learn how easy it is to tweak this basic configuration.","ref":"main.html#quick-start"},{"type":"extras","title":"Configuration - Lapin, a RabbitMQ client for Elixir","doc":"You can configure multiple connections. Each connection is backed by a worker\nwhich can implement a few callbacks to publish/consume messages and handle other\ntype of events from the broker. Each connection can have one or more producers/consumers.\n\nThe default implementation of all callbacks simply returns `:ok`.\n\nYou need to configure a worker module for all connections. To implement a worker\nmodule, define a module and use the `Lapin.Connection` behaviour, then add it\nunder the `module` key in your configuration.\n\nFor details on implementing *Lapin* worker modules check out the `Lapin.Connection`\nbehaviour documentation.\n\nAt a minimum, you need to configure a *module* for each connection, an *exchange*\nfor each producers and a *queue* for each consumer. You can find the complete list\nof connection configuration settings in the in `Lapin.Connection` *config* type specification.\n\nAdvanced consumer/producer behaviour can be configured in two ways.\n\n#","ref":"main.html#configuration"},{"type":"extras","title":"One-shot, static consumer/producer configuration - Lapin, a RabbitMQ client for Elixir","doc":"If you are fine with one shot configuration of your comsumers/producers, you can specify\nany settings from the `Lapin.Consumer`/`Lapin.Producer` `config` type specification\ndirectly in your the corresponding configuration keyword list.\n\nThis is quick and easy way to start.\n\n`lib/example_app/some_worker.ex`:\n\n```elixir\ndefmodule ExampleApp.Worker do\n  use Lapin.Connection\nend\n```\n\n`config/config.exs`:\n\n```elixir\nconfig :lapin, :connections, [\n  [\n    module: ExampleApp.Worker,\n    consumers: [\n      [queue: \"some_queue\", ack: true]\n    ],\n    producers: [\n      [exchange: \"some_exchange\", mandatory: true]\n    ]\n  ]\n]\n```\n\n#","ref":"main.html#one-shot-static-consumer-producer-configuration"},{"type":"extras","title":"Reusable, static or dynamic consumer/producer configuration - Lapin, a RabbitMQ client for Elixir","doc":"If you need to configure a lot of consumers/producers in the same way, you can use a\n`Lapin.Consumer`/`Lapin.Producer` to define reusable configurations. A pattern is simply\na collection of behaviour callbacks bundled in a module, which you can then reuse in any\nconsumer/producer configuration when you need the same kind of interaction pattern.\n\nTo do this, you need to define your producer/consumer module by `use Lapin.Consumer`\nor `use Lapin.Producer` and specify it in your configuration under the *pattern* key.\n\nIn fact `Lapin` bundles a few consumer/producer implementations for the\n*RabbitMQ* [tutorials patterns](https://www.rabbitmq.org/getstarted.html).\n\n`lib/example_app/some_pattern.ex`:\n\n```elixir\ndefmodule ExampleApp.Consumer do\n  use Lapin.Consumer\n\n  def ack(_channel), do: true\n  def consumer_prefetch(_channel), do: 1\nend\n\ndefmodule ExampleApp.Producer do\n  use Lapin.Producer\n\n  def confirm(_channel), do: false\n  def mandatory(_channel), do: true\n  def persistent(_channel), do: true\nend\n```\n\n`config/config.exs`:\n\n```elixir\nconfig :lapin, :connections, [\n  [\n    module: ExampleApp.Worker\n    consumers: [\n      [queue: \"some_queue\", pattern: ExampleApp.Consumer]\n    ],\n    producers: [\n      [exchange: \"some_exchange\", pattern: ExampleApp.Producer]\n    ]\n  ]\n]\n```\n\nSince `Lapin.Consumer` and `Lapin.Producer` are just behaviours of overridable callback\nfunctions, they also allow you to implement any kind of dynamic runtime configuration.\n\nActually, the one-shot static configuration explained earlier is provided by the default\nimplementations, which read settings from the configuration and tries to provide sensible defaults.\n\n#","ref":"main.html#reusable-static-or-dynamic-consumer-producer-configuration"},{"type":"extras","title":"Declaring broker configuration - Lapin, a RabbitMQ client for Elixir","doc":"If you want to declare exchanges and queues with the broker you can do so in the configuration.\n*Lapin* will just declare the specied exchanges, queues and bindings and report any discrepancies\nbetween the configuration and the broker state if there are any.\n\n```elixir\nconfig :lapin, :connections, [\n  module: ExampleApp.Worker,\n  exchanges: [\n    some_exchange: [\n      type: :direct,\n      options: [durable: true],\n      binds: [\n        some_queue: [routing_key: \"some_routing_key\"]\n      ]\n    ]\n  ],\n  queues: [\n    some_queue: [\n      options: [durable: true],\n      binds: [\n        some_exchange: [routing_key: \"some_routing_key\"]\n      ]\n    ]\n  ],\n  ...\n]\n```\n\nExchange declarations support `type` (*default: :direct*), `options` (see `AMQP.Exchange.declare/4`\nfor allowed exchange types and options) and `binds`, which is a keyword list of queue names and\ndeclare arguments (see `AMQP.Queue.bind/4` for allowed arguments).\n\nQueue declarations support `options` (see `AMQP.Queue.declare/3` for allowed options) and `binds`,\nwhich is a keyword list of exchange names and declare arguments (see `AMQP.Exchange.bind/4` for\nallowed arguments).","ref":"main.html#declaring-broker-configuration"},{"type":"extras","title":"Usage - Lapin, a RabbitMQ client for Elixir","doc":"#","ref":"main.html#usage"},{"type":"extras","title":"Consuming messages - Lapin, a RabbitMQ client for Elixir","doc":"Once you have completed your configuration, connections will be automatically established and channels\nwill start receiving messages published on the queues they are consuming.\n\nYou can handle received messages by overriding the `Lapin.Connection` `handle_deliver/2` callback.\nThe default implementation simply logs messages and returns `:ok`.\n\n```elixir\ndefmodule ExampleApp.Worker do\n  use Lapin.Connection\n\n  def handle_deliver(channel, message) do\n    Logger.debug fn -> \"received #{inspect message} on #{inspect channel}\" end\n    :ok\n  end\nend\n```\n\nSince messages for all consumers on the same connection are received by the same worker module,\nto dispatch messages to different handling logic you can pattern match on the `Channel.config` map,\nwhich contains message routing information.\n\n```elixir\ndefmodule ExampleApp.Worker do\n  use Lapin.Connection\n\n  def handle_deliver(%Channel{exchange: \"a\", queue: \"b\"} = channel, message) do\n    Logger.debug fn -> \"received #{inspect message} on #{inspect channel}\" end\n    :ok\n  end\n\n  def handle_deliver(%Channel{exchange: \"c\", queue: \"d\"} = channel, message) do\n    Logger.debug fn -> \"received #{inspect message} on #{inspect channel}\" end\n    :ok\n  end\nend\n```\n\nMessages are considered to be successfully consumed if the `Lapin.Connection` `handle_deliver/2`\ncallback returns `:ok`. See the callback documentation for a complete list of possible values you\ncan return to signal message acknowledgement and rejection to the broker.\n\n#","ref":"main.html#consuming-messages"},{"type":"extras","title":"Publishing messages - Lapin, a RabbitMQ client for Elixir","doc":"To publish messages you can use the `publish` function injected in your worker module by\n`use Lapin.Connection`, or directly call `Lapin.Connection.publish/5` by passing your worker module\nas the connection argument.\n\n`config/config.exs`:\n\n```elixir\nconfig :lapin, :connections, [\n  [\n    module: ExampleApp.Worker,\n    producers: [\n      [exchange: \"some_exchange\"]\n    ]\n  ]\n]\n```\n\nUsing the worker module implementation:\n\n```elixir\n:ok = ExampleApp.Worker.publish(\"some_exchange\", \"routing_key\", \"payload\", [])\n```\n\nVia `Lapin.Connection` by passing the worker module as the connection:\n\n```elixir\n:ok = Lapin.Connection.publish(ExampleApp.Worker, \"some_exchange\", \"routing_key\", \"payload\", [])\n```\n\nIf you are starting a `Lapin.Connection` manually, you can also pass the connection pid:\n\n```elixir\n{:ok, pid} = Lapin.Connection.start_link([\n  module: ExampleApp.Worker,\n  producers: [\n    [exchange: \"some_exchange\"]\n  ]\n])\n\n:ok = Lapin.Connection.publish(pid, \"some_exchange\", \"routing_key\", %Lapin.Message{}, [])\n```\n\n#","ref":"main.html#publishing-messages"},{"type":"extras","title":"Message payload encoding - Lapin, a RabbitMQ client for Elixir","doc":"Message payload is assumed to be `binary` by default, and is sent and received\nunaltered by your code. However, Lapin can handle message encoding and decoding\nfor you.\n\nAutomatic encoding of the message payload is done by passing a data type other\nthan binary as the `payload` argument to the publish methods. And by providing\nan implementation of the `Lapin.Message.Payload` protocol for your data type.\n\nWhen consuming, the `Lapin.Connection` `payload_for/2` callback of the worker\nmodule allows you to return an instance of the data type you want to perform\nmessage decoding into. Again, an implementation of the `Lapin.Message.Payload`\nprotocol is required for your custom `payload` data type (e.g. a struct).\n\nThe `Lapin.Message.Payload` protocol documentation explains how to implement\nthe required functions, and provides an example JSON to struct implementation.","ref":"main.html#message-payload-encoding"}],"content_type":"text/markdown"}